package com.sun.org.glassfish.gmbal;

import javax.management.ObjectName;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.security.PrivilegedExceptionAction;
import java.lang.reflect.Method;
import com.sun.org.glassfish.gmbal.util.GenericConstructor;

public final class ManagedObjectManagerFactory
{
    private static GenericConstructor<ManagedObjectManager> objectNameCons;
    private static GenericConstructor<ManagedObjectManager> stringCons;
    
    private ManagedObjectManagerFactory() {
    }
    
    public static Method getMethod(final Class<?> cls, final String name, final Class<?>... types) {
        try {
            return (Method)AccessController.doPrivileged(new PrivilegedExceptionAction<Method>() {
                @Override
                public Method run() throws Exception {
                    return cls.getDeclaredMethod(name, types);
                }
            });
        }
        catch (PrivilegedActionException ex) {
            throw new GmbalException("Unexpected exception", ex);
        }
        catch (SecurityException exc) {
            throw new GmbalException("Unexpected exception", exc);
        }
    }
    
    public static ManagedObjectManager createStandalone(String domain) {
        ManagedObjectManager result = (ManagedObjectManager)ManagedObjectManagerFactory.stringCons.create(new Object[] { domain });
        if (result == null) {
            return ManagedObjectManagerNOPImpl.self;
        }
        return result;
    }
    
    public static ManagedObjectManager createFederated(ObjectName rootParentName) {
        ManagedObjectManager result = (ManagedObjectManager)ManagedObjectManagerFactory.objectNameCons.create(new Object[] { rootParentName });
        if (result == null) {
            return ManagedObjectManagerNOPImpl.self;
        }
        return result;
    }
    
    public static ManagedObjectManager createNOOP() {
        return ManagedObjectManagerNOPImpl.self;
    }
    
    static {
        ManagedObjectManagerFactory.objectNameCons = new GenericConstructor(ManagedObjectManager.class, "com.sun.org.glassfish.gmbal.impl.ManagedObjectManagerImpl", new Class[] { ObjectName.class });
        ManagedObjectManagerFactory.stringCons = new GenericConstructor(ManagedObjectManager.class, "com.sun.org.glassfish.gmbal.impl.ManagedObjectManagerImpl", new Class[] { String.class });
    }
}
